import type { ColorValue, StyleProp, TextStyle } from 'react-native'
import type { SyntaxHighlighterProps } from 'src/types/shiki'
import type { ReactStyle } from '../../utils/style-transformer'
import React, { useMemo } from 'react'
import { Platform, ScrollView, StyleSheet, Text, View } from 'react-native'
import { useSyntaxHighlighter } from '../../hooks/useSyntaxHighlighter'
import { getRNStylesFromShikiStyle } from '../../utils/style-transformer'

const monospaceFont = Platform.select({
  ios: 'Menlo',
  android: 'monospace',
  default: 'monospace',
})

interface TokenStyle extends Omit<TextStyle, 'color'> {
  color?: ColorValue
  fontFamily?: string
}

interface ThemeStyles {
  token: TokenStyle
  [key: string]: TokenStyle
}

const baseStyles = StyleSheet.create({
  scrollView: {
    flex: 1,
    minHeight: 20,
    backgroundColor: '#24292e', // GitHub Dark default background
  },
  lineContainer: {
    flexDirection: 'row',
    flexWrap: 'wrap',
    paddingHorizontal: 16,
    paddingVertical: 8,
  },
  token: {
    fontFamily: monospaceFont,
    fontSize: 14,
    color: '#e1e4e8', // GitHub Dark default text color
  },
  container: {
    flex: 1,
  },
})

const SyntaxHighlighter: React.FC<SyntaxHighlighterProps> = ({ text, language, languageId, theme, fontSize = 14 }) => {
  const styles = useMemo(() => {
    const themeStyles = getRNStylesFromShikiStyle(theme as ReactStyle)
    return StyleSheet.create<ThemeStyles>({
      ...themeStyles,
      token: {
        ...baseStyles.token,
        fontSize,
      },
    })
  }, [theme, fontSize])

  const { tokens } = useSyntaxHighlighter({
    text,
    language,
    languageId,
    theme,
  })

  const renderToken = (token: any, index: number, lineIndex: number) => {
    // Start with the base token style
    const tokenStyles: StyleProp<TokenStyle>[] = [styles.token]

    // Add token-specific styles from stylesheet based on scope
    const tokenScopes = Array.isArray(token.scope) ? token.scope : [token.scope]
    tokenScopes.forEach((scope: string) => {
      if (styles[scope]) {
        tokenStyles.push(styles[scope])
      }
    })

    // Add token-specific styles from the token itself
    if (token.color) {
      tokenStyles.push({ color: token.color })
    }
    if (token.fontStyle === 'italic') {
      tokenStyles.push({ fontStyle: 'italic' })
    }
    if (token.fontWeight === 'bold') {
      tokenStyles.push({ fontWeight: 'bold' })
    }

    const finalStyle = StyleSheet.flatten(tokenStyles)

    return (
      <Text key={`${lineIndex}-${index}`} style={finalStyle}>
        {token.content.replace(/ /g, '\u00A0')}
      </Text>
    )
  }

  const renderLine = (line: any[], lineIndex: number) => (
    <View key={lineIndex} style={baseStyles.lineContainer}>
      {line.map((token, tokenIndex) => renderToken(token, tokenIndex, lineIndex))}
    </View>
  )

  return (
    <ScrollView horizontal showsHorizontalScrollIndicator={Platform.OS !== 'web'} style={baseStyles.scrollView} contentContainerStyle={baseStyles.container}>
      {tokens.map((line, index) => renderLine(line, index))}
    </ScrollView>
  )
}

export default SyntaxHighlighter
